// create.cpp : Example of creating and populating a new AMTree
//

#include "stdafx.h"
#include <fstream.h>

#include "AMTreeDLL.h"		//It defines IAMTree, that is the interface for the AMTree component
#include "AMTreeDLL_i.c"	//Definition of IIDs and CLSIDs constants for the AMTree component

#include "object.h"			//It defines the interface IMTobject implemented by any obejct managed by AMTrees
#include "object_i.c"		//It defines the IID of the interface IAMTobject

#include "HV1_i.c"			//this is the definition for CLSIDs for the class HV1Object;
							//It implements the interface IAMTobject. HV1 obejcts encapsulate
							//vectors of 45 doubles. Distance is measured by using the eucledian distance.



void main(int argc, char* argv[])
{
	CoInitialize(NULL);						//initialize COM

	IAMTree*am_tree=NULL;					// reference to the component implementing the AMTree
	CoCreateInstance(						//create an instance af AMTree 
			CLSID_AMTree,					//GUID of the Class implementing the AMTree. It is defined in AMTreeDLL_i.c
			NULL,
			CLSCTX_SERVER,
			IID_IAMTree,					//GUID of the interface IAMTree. It is defined in AMTreeDLL_i.c
			(void **)&am_tree);

	am_tree->Create(						//create a new index
		(unsigned char *)"..\\HV1index",	//directory where the index will be created
		10000,								//maximum distance in the used bounded metric space
		CLSID_HV1Object						//CLSID of the object managed by this AMTree; it is defined in HV1_i.c
		);					

// The whole set of vectors is contained in the file Data.dat in the directory HV1dataset
	fstream *dataset=new fstream("..\\HV1dataset\\Data.dat",ios::in, filebuf::sh_read);
	cout << "inserting objects in the index..." << endl;
	double vect[45];						//vector read from the dataset
	for(int i=0; i<10000;i++){				// We insert 10000 vectors.
		if(!(i%1000)){
			cout << "*";
			cout.flush();
		}
		for(int dim=0; dim < 45; dim++)
			*dataset >> vect[dim];
		IMTobject *pObj;					//now we have the vector. Let's create an object that encapsulate it.
		CoCreateInstance(					// HV1Objcts encapsulate vectors of 45 dimensions.
			CLSID_HV1Object,				//CLSSID of the class implementing HV1Object
			NULL,				
			CLSCTX_SERVER,
			IID_IMTobject,					//GUID of the interface IMTobject
			(void **)&pObj);
		pObj->initialize((unsigned char *)vect);//The object is uninitialized. We initialize it
		am_tree->Insert(					// we can insert the object in the index
			i,								//object identifier
			pObj							//the object
		);
		pObj->Release();					//The object can be safely released
	}
	cout << endl <<"finished inserting objects in the index..." << endl;
	dataset->close();						//dataset file no longer needed
	am_tree->generateStatistics();			//The index was populated with 10000 new object.
											//We should update the statistics to have better performances
	am_tree->Release();						//the AMTree can be released
}
