// randomInsert.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <fstream.h>
#include <time.h>
#include <stdlib.h>
#include <math.h>

#include "AMTreeDLL.h"		//It defines IAMTree, that is the interface for the AMTree component
#include "AMTreeDLL_i.c"	//Definition of IIDs and CLSIDs constants for the AMTree component

#include "object.h"			//It defines the interface IMTobject implemented by any obejct managed by AMTrees
#include "object_i.c"		//It defines the IID of the interface IAMTobject

#include "Histo144_i.c"			//this is the definition for CLSIDs for the class Histo144Object;
							//It implements the interface IAMTobject. Histo144 obejcts encapsulate
							//vectors of 144 doubles. Distance is measured by using the eucledian distance.



void main(int argc, char* argv[])
{

	CoInitialize(NULL);						//initialize COM

	IMTobject *pObj;
	CoCreateInstance(					// Histo144Objcts encapsulate vectors of 45 dimensions.
		CLSID_Histo144Object,				//CLSSID of the class implementing Histo144Object
		NULL,				
		CLSCTX_SERVER,
		IID_IMTobject,					//GUID of the interface IMTobject
		(void **)&pObj);
	int size;
	pObj->compressedSize(&size);
	int dimension= size/sizeof(double);


	IAMTree*am_tree=NULL;					// reference to the component implementing the AMTree
	CoCreateInstance(						//create an instance af AMTree 
			CLSID_AMTree,					//GUID of the Class implementing the AMTree. It is defined in AMTreeDLL_i.c
			NULL,
			CLSCTX_SERVER,
			IID_IAMTree,					//GUID of the interface IAMTree. It is defined in AMTreeDLL_i.c
			(void **)&am_tree);

	am_tree->Create(						//create a new index
		(unsigned char *)"..\\Histo144index",	//directory where the index will be created
		sqrt(dimension),								//maximum distance in the used bounded metric space
		CLSID_Histo144Object						//CLSID of the object managed by this AMTree; it is defined in Histo144_i.c
		);					

// We generate casually vectors
	cout << "inserting objects in the index..." << endl;
	srand( (unsigned)time( NULL ) );
	double *vect;
	vect=new double[dimension];						//vector
	for(int i=0; i<10000;i++){				// We insert 10000 vectors.
		if(!(i%1000)){
			cout << "*";
			cout.flush();
		}
		for(int dim=0; dim < dimension; dim++)
			vect[dim]=((double)rand()/(double)RAND_MAX);
											//now we have the vector. Let's create an object that encapsulate it.
		pObj->initialize((unsigned char *)vect);//The object is uninitialized. We initialize it
		am_tree->Insert(					// we can insert the object in the index
			i,								//object identifier
			pObj							//the object
		);
	}
	cout << endl <<"finished inserting objects in the index..." << endl;
	pObj->Release();					//The object can be safely released
	am_tree->generateStatistics();			//The index was populated with 10000 new object.
											//We should update the statistics to have better performances
	am_tree->Release();						//the AMTree can be released
}
