// search.cpp : Defines the entry point for the console application.
//


#include "stdafx.h"
#include <fstream.h>

#include "AMTreeDLL.h"		//It defines IAMTree, that is the interface for the AMTree component
#include "AMTreeDLL_i.c"	//Definition of IIDs and CLSIDs constants for the AMTree component

#include "object.h"			//It defines the interface IAMTobject implemented by any obejct managed by AMTrees
#include "object_i.c"		//It defines the IID of the interface IAMTobject

#include "HV1_i.c"			//this is the definition for CLSIDs for the class HV1Object;
							//It implements the interface IMTobject. HV1 obejcts encapsulate
							//vectors of 45 doubles. Distance is measured by using the eucledian distance.

#include"approximationDefs.h"	//it contains definition od names of approximation methods


void printCursor(IAMTree *am_tree, IMTobject *pObj){
	BOOL b;
	while(am_tree->next(&b),b){				//scan the whole result set
		unsigned long oid;
		double dist;
		am_tree->getObjectId(&oid);			//get the current object id
		am_tree->getObjectDistance(&dist);	//get the distance of this obejct from the query object
		IMTobject *tobj;					
		am_tree->getObject(&tobj);			//get the current object
		double cdist;
		tobj->distance(pObj,&cdist);		//compute the distance of this object from the query object
											//it should be equal to dist
		tobj->Release();					//current object can be released
		cout << "OID:"<< oid << " distance stored:" << dist << " distance explicitely computed:" << cdist << endl;
	}

}


void main(int argc, char* argv[])
{
	CoInitialize(NULL);						//initialize COM

	IAMTree*am_tree=NULL;					// reference to the component implementing the AMTree
	CoCreateInstance(						//create an instance af AMTree 
			CLSID_AMTree,					//GUID of the Class implementing the AMTree. It is defined in AMTreeDLL_i.c
			NULL,
			CLSCTX_SERVER,
			IID_IAMTree,					//GUID of the interface IAMTree. It is defined in AMTreeDLL_i.c
			(void **)&am_tree);


	am_tree->Open(							//opening the index
		(unsigned char *) "..\\HV1index",	//directory where the index is stored
		CLSID_HV1Object						//CLSID of the object managed by this AMTree; it is defined in HV1_i.c
		);
//	am_tree->generateStatistics();			//generate statistics (not really needed)
	fstream *dataset=new fstream("..\\HV1dataset\\Data.dat",ios::in, filebuf::sh_read);
	double vect[45];						//vector read from the dataset
	for(int dim=0; dim < 45; dim++)			//reading a vector to use as query
		*dataset >> vect[dim];
	IMTobject *pObj;						//now we have the vector. Let's create an object that encapsulate it.
	CoCreateInstance(						// HV1Objcts encapsulate vectors of 45 dimensions.
		CLSID_HV1Object,					//CLSSID of the class implementing HV1Object
		NULL,				
		CLSCTX_SERVER,
		IID_IMTobject,						//GUID of the interface IMTobject
		(void **)&pObj);
	pObj->initialize((unsigned char *)vect);//The object is uninitialized. We initialize it
/////////////////////////////////////
	am_tree->resetCounters();				//reset counters. 
											//This resets the number of disk reads and distance computation countes.
											//It can be used for objective performance evaluation and comparison
	am_tree->setApproximationMethod(APPROX_METHOD_EXACT);	//first we execute an exact search
	am_tree->NNSearch(pObj,10);				//we execute an exact 10 nearest neighbors search
	int IOreads;
	int comDists;
	am_tree->getIOReads(&IOreads);			// get the number of disk reads
	am_tree->getComputedDistances(&comDists);// get the number of distance computations
	cout << endl << "exact 10 NN search:" << endl;
	cout << "Disk reads:" << IOreads << " Distance computations:" << comDists << endl;
	printCursor(am_tree,pObj);				//print the result set
/////////////////////////////////////
	am_tree->resetCounters();				//reset counters. 
	am_tree->setApproximationMethod(APPROX_METHOD_PROXIMITY);	//we execute an approximate search using the proximity method
	double thresholds[]={0.03};
	am_tree->setApproximationThresholds(	//set the approximation thresholds !!THIS IS COMPULSORY!!
		thresholds,							//array of thresholds
		1);									//the proximity method uses just one threshold
	am_tree->NNSearch(pObj,10);				//we execute an approximate 10 nearest neighbors search
	IOreads;
	comDists;
	am_tree->getIOReads(&IOreads);			// get the number of disk reads
	am_tree->getComputedDistances(&comDists);// get the number of distance computations
	cout << endl << "approximate 10 NN search with the proximity method and threshold" << thresholds[0] << endl;
	cout << "Disk reads:" << IOreads << " Distance computations:" << comDists << endl;
	printCursor(am_tree,pObj);				//print the result set
/////////////////////////////////////
	am_tree->resetCounters();				//reset counters. 
	am_tree->setApproximationMethod(APPROX_METHOD_DISTRIBUTION);	//we execute an approximate search using the distribution method
	thresholds[0]=0.3;
	am_tree->setApproximationThresholds(	//set the approximation thresholds !!THIS IS COMPULSORY!!
		thresholds,							//array of thresholds
		1);									//the distribution method uses just one threshold
	am_tree->NNSearch(pObj,10);				//we execute an approximate 10 nearest neighbors search
	IOreads;
	comDists;
	am_tree->getIOReads(&IOreads);			// get the number of disk reads
	am_tree->getComputedDistances(&comDists);// get the number of distance computations
	cout << endl << "approximate 10 NN search with the distribution method and threshold" << thresholds[0] << endl;
	cout << "Disk reads:" << IOreads << " Distance computations:" << comDists << endl;
	printCursor(am_tree,pObj);				//print the result set
/////////////////////////////////////
	am_tree->resetCounters();				//reset counters. 
	am_tree->setApproximationMethod(APPROX_METHOD_PROXIMITY);	//we execute an approximate search using the proximity method
	thresholds[0]=0.3;
	am_tree->setApproximationThresholds(	//set the approximation thresholds !!THIS IS COMPULSORY!!
		thresholds,							//array of thresholds
		1);									//the proximity method uses just one threshold
	am_tree->RangeSearch(pObj,4200);				//we execute an approximate range search
	IOreads;
	comDists;
	am_tree->getIOReads(&IOreads);			// get the number of disk reads
	am_tree->getComputedDistances(&comDists);// get the number of distance computations
	cout << endl << "approximate range search with the proximity method and threshold" << thresholds[0] << endl;
	cout << "Disk reads:" << IOreads << " Distance computations:" << comDists << endl;
	printCursor(am_tree,pObj);				//print the result set

	
	pObj->Release();						//The object can be safely released
	am_tree->Release();						//The AMTree can be released

}

